package net.java.dev.esjug.util.ejb3.domain;

import java.io.Serializable;

/**
 * Optional interface for domain objects that implement equals() and
 * hashCode() using an Universal Unique Identifier (UUID).
 * 
 * </p><p>
 * It is very common to have domain objects in collections such as HashSet,
 * which assume that the methods equals() and hashCode() have been implemented
 * in a way that all instances that represent the same object return the same
 * value as hashCode() and are equals according to equals().
 * 
 * </p><p>
 * Many implementations of equals() and hashCode() have already been suggested:
 * using properties of the domain object that univocally identifies an object;
 * using the persistence id (i.e. the primary key); etc. Each has its own
 * advantages and disadvantages. 
 * 
 * </p><p>
 * The solution proposed by this class is the utilization of an Universal Unique
 * Identifier (UUID). This interface provides the methods to obtain the UUID.
 * There is a standard implementation for this interface: DomainObjectSupport.
 * 
 * @see net.java.dev.esjug.util.ejb3.domain.DomainObjectSupport
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
public interface DomainObject extends Serializable {
	/**
	 * Obtains the Universal Unique Identifier.
	 * 
	 * @return O UUID.
	 */
	String getUuid();
}
